// src/cairo_elements.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef CAIRO_ELEMENTS_HH
#define CAIRO_ELEMENTS_HH 1


#include <sgpemv2/thread.hh>
#include <sgpemv2/process.hh>

#include <cairo.h>

namespace sgpem
{

  struct Point
  {
    Point(float X, float Y) 
      : x(X), y(Y) {}
    float x;
    float y;
  };

  struct Rectangle
  { 
    Rectangle(float X0, float Y0, float W, float H) 
      : x0(X0), y0(Y0), w(W), h(H) {}
    float x0;
    float y0;
    float w;
    float h;
  };

  struct Color
  {
    Color(float R, float G, float B)
      : r(R), g(G), b(B) {}
    float r;
    float g;
    float b;
  };


  class CairoElements
  {
  public:
    CairoElements(cairo_t* const ctx);

    void draw_3dsphere(const Point& center, float radius, const Color& cl);
    void draw_3dcube(const Rectangle& area, const Color& cl, const float x_percent, const float y_percent);
    void draw_container(const Rectangle& area);
    void draw_expandable(const Rectangle& area, bool expanded = false);

  private:
    cairo_t*  _ctx;
  };

} //~ namespace sgpem


#endif //~ CAIRO_ELEMENTS_HH
